/* ========================================================================
 * Copyright (C) 2000-2004  Technische Universitaet Muenchen
 * License: GPL v2 (or later)
 *
 * For further information please contact the DWARF team at
 * <dwarf-gnu@augmentedreality.de>
 *
 * AutoTune.h
 * Daniel Pustka <pustka@in.tum.de> May 2004
 * Distributed Wearable Augmented Reality Framework - www.augmentedreality.de
 * For documentation and instructions, see the DWARF documentation website at
 * http://wwwbruegge.in.tum.de/projects/dwarf/doc/
 *
 * For further questions about DWARF, please contact us at
 * <dwarf@augmentedreality.de>
 * For questions about this particular service, contact
 * Daniel Pustka <pustka@in.tum.de>
 *
 * $Id: AutoTuner.h,v 1.2 2004/08/14 23:32:39 pustka Exp $
 */

/**
 * @file AutoTuner.h
 * @brief Automatically tunes an EKF for a given dataset
 *
 * @author Daniel Pustka <pustka@in.tum.de>
 * @ingroup PoseTool
 */

#ifndef __AutoTuner_H_
#define __AutoTuner_H_

// autoconf generated file with configuration information
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <PoseEkf.h>
#include <qthread.h>
#include "PoseSequence.h"

#ifdef HAVE_GSL
#include <gsl/gsl_multimin.h>
#endif // HAVE_GSL

class AutoTuner
   {
   protected:
      PoseSequenceSubCollection m_Sequences;

      PoseEKF::MotionModel m_MoMo;
      int m_nPredict;
      bool m_bPosXYZIndependent;

#ifdef HAVE_GSL
      static double my_opt_f( const gsl_vector* v, void* par );
#endif
      
   public:
      AutoTuner( const PoseSequenceSubCollection& Sources,
         const PoseEKF::MotionModel& MoMo, int nPredict, bool bPosXYZIndependent );

      /** compute an error for the current motion model */
      double ComputePredictionError( double& fAvgErr );

#ifdef HAVE_GSL
      /** optimize the current motion model */
      void OptimizePN();
#endif

      const PoseEKF::MotionModel& GetMotionModel() const
         { return m_MoMo; }
   };

   
#endif //__EkfWindow_H_
