/***************************************************************************
 *                         EkfMeasure.h  -  description
 *                            -------------------
 *   begin                : 2004/05/21
 *   copyright            : (C) 2004 by Daniel Pustka
 *   email                : pustka@in.tum.de
 *
 *   $Id: EkfMeasure.h,v 1.1 2004/06/07 13:34:29 pustka Exp $
 *   $Revision: 1.1 $
 *
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef __EKFMEASURE_H__
#define __EKFMEASURE_H__

#include "Ekf.h"

/**
 * Implements a measurement function that just reads elements out of the state vector
 * To be used together with the generic ExtendedKalmanFilter
 */
class IdentityMeasurement : public ExtendedKalmanFilter::MeasurementFunction
   {
   protected:
      int m_nVariables;

   public:
      /**
       * constructor
       * @param nVariables number of state vector variables to return
       */
      IdentityMeasurement( int nVariables );

      /**
       * Computes the predicted measurement pOut from the predicted state pIn
       */
      TNT::Array2D<double> PredictMeasurement( const double* pIn, double* pOut ) const;
   };


/**
 * Implements a measurement function that normalizes a quaternion
 * To be used together with the generic ExtendedKalmanFilter
 */
class QuaternionMeasurement : public ExtendedKalmanFilter::MeasurementFunction
   {
   public:
      /**
       * Computes the predicted measurement pOut from the predicted state pIn
       */
      TNT::Array2D<double> PredictMeasurement( const double* pIn, double* pOut ) const;
   };


/**
 * Combines multiple measurement functions. Useful when we have a combined covariance.
 */
class CombinedMeasurement : public ExtendedKalmanFilter::MeasurementFunction
   {
   protected:
      struct MFInfo
         {
         int nMeasurementSize;
         std::pair<int, int> InRange;

         MFInfo( int _nMeasurementSize, std::pair<int, int> _IR )
            { nMeasurementSize = _nMeasurementSize; InRange = _IR; }
         };

      auto_vector<ExtendedKalmanFilter::MeasurementFunction> m_MFs;
      std::vector<MFInfo> m_MFInfos;

   public:
      /**
       * Constructor. See ExtendedKalmanFilter::Measure for an explanation of the parameters.
       */
      CombinedMeasurement(
         int nMeasurementSize1, std::pair<int, int> InRange1,
         std::auto_ptr<ExtendedKalmanFilter::MeasurementFunction> pMF1,
         int nMeasurementSize2, std::pair<int, int> InRange2,
         std::auto_ptr<ExtendedKalmanFilter::MeasurementFunction> pMF2 );

      /**
       * Computes the predicted measurement pOut from the predicted state pIn
       */
      TNT::Array2D<double> PredictMeasurement( const double* pIn, double* pOut ) const;
   };

#endif
