/***************************************************************************
 *                         EkfPredict.h  -  description
 *                            -------------------
 *   begin                : 2004/05/21
 *   copyright            : (C) 2004 by Daniel Pustka
 *   email                : pustka@in.tum.de
 *
 *   $Id: EkfPredict.h,v 1.1 2004/06/07 13:34:29 pustka Exp $
 *   $Revision: 1.1 $
 *
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef __EKFPREDICT_H__
#define __EKFPREDICT_H__

#include "Ekf.h"

/**
 * Implements a polynomial predictor using the first n derivations of a vector.
 * To be used as a time update function for the generic ExtendedKalmanFilter.
 */
class PolynomialPredictor : public ExtendedKalmanFilter::TimeUpdateFunction
   {
   protected:
      int m_nVariables;
      int m_nDerivations;

   public:
      /* constructor
       * @param nVariables size of vector to predict
       * @param nDerivations number of derivations to use
       */
      PolynomialPredictor( int nVariables, int nDerivations );

      /**
       * Predicts state pOut from state pIn
       */
      virtual TNT::Array2D<double>
         Predict( double fTimeDelta, const double* pIn, double* pOut );
   };


/**
 * Implements a predictor for quaternions
 * using the logarithm of the difference quaternion and it's derivations.
 * To be used as a time update function for the generic ExtendedKalmanFilter.
 */
class QuaternionPredictor : public ExtendedKalmanFilter::TimeUpdateFunction
   {
   protected:
      int m_nDerivations;

   public:
      /* constructor
       * @param nDerivations number of derivations to use
       */
      QuaternionPredictor( int nDerivations );

      /**
       * Predicts state pOut from state pIn
       */
      virtual TNT::Array2D<double>
         Predict( double fTimeDelta, const double* pIn, double* pOut );
   };

#endif
