/* ========================================================================
 * Copyright (C) 2000-2004  Technische Universitaet Muenchen
 * License: GPL v2 (or later)
 *
 * For further information please contact the DWARF team at
 * <dwarf-gnu@augmentedreality.de>
 *
 * PoseEKF.h
 * Daniel Pustka <pustka@in.tum.de> May 2004
 * Distributed Wearable Augmented Reality Framework - www.augmentedreality.de
 * For documentation and instructions, see the DWARF documentation website at
 * http://wwwbruegge.in.tum.de/projects/dwarf/doc/
 *
 * For further questions about DWARF, please contact us at
 * <dwarf@augmentedreality.de>
 * For questions about this particular service, contact
 * Daniel Pustka <pustka@in.tum.de>
 *
 * $Id: PoseEkf.h,v 1.1 2004/06/07 13:34:29 pustka Exp $
 */

#ifndef __POSEEKF_H__
#define __POSEEKF_H__

#include "Ekf.h"
#include <ubitrack/EasyPoseData.h>
#include <list>
#include <vector>


/** An ExtendedKalmanFilter specialized for PoseData handling */
class PoseEKF : public ExtendedKalmanFilter
   {
   public:

      /** describes a motion model  for the PoseEKF */
      class MotionModel
         {
         protected:
            int m_nPosDerivations;
            int m_nOriDerivations;
            std::vector<double> m_PosProcessNoise;
            std::vector<double> m_OriProcessNoise;

         public:
            MotionModel( int nPosDerivations = -1, int nOriDerivations = -1 );

            /** -1 for no position */
            void SetPosDerivations( int nPosDerivations );

            /** -1 for no orientation */
            void SetOriDerivations( int nPosDerivations );

            int GetPosDerivations() const
               { return m_nPosDerivations; }

            int GetOriDerivations() const
               { return m_nOriDerivations; }

            void SetPosProcessNoise( int nComponent, int nDerivation, double v )
               { m_PosProcessNoise[ 3 * nDerivation + nComponent ] = v; }

            void SetOriProcessNoise( int nDerivation, double v )
               { m_OriProcessNoise[ nDerivation ] = v; }

            double GetPosProcessNoise( int nComponent, int nDerivation ) const
               { return m_PosProcessNoise[ 3 * nDerivation + nComponent ]; }

            double GetOriProcessNoise( int nDerivation ) const
               { return m_OriProcessNoise[ nDerivation ]; }
         };

   protected:
      MotionModel m_MoMo;
      int m_nPosStateSize;
      bool m_bInitialized;
      DWARF::Time m_Time;

      void Init( const DWARF::PoseData& P );
      TNT::Array2D<double> GetV( const double* pOrientation, bool bWithPos );
      /** convert strange multiplicative 3x3/6x6 covariance to additive 4x4/7x7 covariance */
      TNT::Array2D<double> Pose2KFCov( const DWARF::PoseData& P, bool bWithPos );

   public:
      /** initializes the EKF with the given motion model */
      PoseEKF( const MotionModel& MoMo );

      /**
       * integrate a measurement
       * @param P the new pose
       * @param nPosDerivation > 0 if the position part contains a derivation.
       * @param nOriDerivation > 0 if the orientation part contains a derivation.
       */
      void AddPose( const DWARF::PoseData& P, int nPosDerivation = 0, int nOriDerivation = 0 );

      /** predict a pose */
      DWARF::PoseData PredictPose( const DWARF::Time& PTime );
   };

#endif
