/**
 * @file timeutil.cpp
 *
 * Utility classes and functions to simplify working with the
 * DWARF framework under C++. Time conversion routines
 *
 * @author Daniel Pustka
 */

/*
 * Distributed Wearable Augmented Reality Framework - www.augmentedreality.de
 * $Id: TimeUtil.cpp,v 1.2 2004/08/14 23:12:15 pustka Exp $
 * $Revision: 1.2 $
 */

#include <TimeUtil.h>

DWARF::Time DWARF::operator-( const DWARF::Time& a, const DWARF::Time& b )
  {
  DWARF::Time d;
  d.seconds = a.seconds - b.seconds;
  int nMicroSeconds = static_cast<int>( a.microseconds ) - static_cast<int>( b.microseconds );
  if ( nMicroSeconds < 0 )
    {
    nMicroSeconds += 1000000;
    d.seconds--;
    }
  d.microseconds = nMicroSeconds;
  return d;
  }


DWARF::Time DWARF::operator+( const DWARF::Time& a, const DWARF::Time& b )
  {
  DWARF::Time d;
  d.seconds = a.seconds + b.seconds;
  d.microseconds = a.microseconds + b.microseconds;
  if ( d.microseconds >= 1000000 )
    {
    d.microseconds -= 1000000;
    d.seconds++;
    }
  return d;
  }


std::ostream& DWARF::operator<<( std::ostream& os, const DWARF::Time& t )
  {
  os << t.seconds << '.' << t.microseconds;
  return os;
  }


std::istream& DWARF::operator>>( std::istream& is, DWARF::Time& t )
  {
  is >> t.seconds;
  is.ignore();
  is >> t.microseconds;
  return is;
  }

