/**
 * @file timeutil.h
 *
 * Utility classes and functions to simplify working with the
 * DWARF framework under C++. Time conversion routines
 *
 * @author Daniel Pustka
 */

/*
 * Distributed Wearable Augmented Reality Framework - www.augmentedreality.de
 * $Id: TimeUtil.h,v 1.2 2004/08/14 23:12:15 pustka Exp $
 * $Revision: 1.2 $
 */

#ifndef __TIMEUTIL_H_
#define __TIMEUTIL_H_

#include <DWARF/DwarfCommon.h>
#include <iostream>

namespace DWARF
  {
  // namespace Util -- doesn't work. operators must be in the same namespace
  // as the type of one operand

  // basic comparison operators
  static inline bool operator==( const Time& a, const Time& b )
    { return a.seconds == b.seconds && a.microseconds == b.microseconds; }

  static inline bool operator<( const Time& a, const Time& b )
    { return a.seconds < b.seconds || ( a.seconds == b.seconds && a.microseconds < b.microseconds ); }

  // derived comparison operators
  static inline bool operator!=( const Time& a, const Time& b )
    { return !(a==b); }

  static inline bool operator<=( const Time& a, const Time& b )
    { return a<b || a==b; }

  static inline bool operator>( const Time& a, const Time& b )
    { return !(a<b) && a!=b; }

  static inline bool operator>=( const Time& a, const Time& b )
    { return !(a<b); }

  /** computes the sum of two times */
  Time operator+( const Time&, const Time& );

  /**
   * @brief subtraction of times:
   * Warning: Time only contains unsigned fields, therefore the result
   * should better not be negative!
   */
  Time operator-( const Time&, const Time& );

  /** output a time as <seconds>.<microseconds> */
  std::ostream& operator<<( std::ostream& os, const Time& t );

  /** read a time from <seconds>.<microseconds> */
  std::istream& operator>>( std::istream& is, Time& t );


  namespace Util
    {
    // some useful constant times
    static const Time minTime = { 0, 0 };
    static const Time maxTime = { 0xFFFFFFFF, 0xFFFFFFFF };
    }
  }

#endif
