/* ========================================================================
 * Copyright (C) 2000-2004  Technische Universitaet Muenchen
 * License: GPL v2 (or later)
 *
 * For further information please contact the DWARF team at
 * <dwarf-gnu@augmentedreality.de>
 *
 * DerivThread.h
 * Daniel Pustka <pustka@in.tum.de> May 2004
 * Distributed Wearable Augmented Reality Framework - www.augmentedreality.de
 * For documentation and instructions, see the DWARF documentation website at
 * http://wwwbruegge.in.tum.de/projects/dwarf/doc/
 *
 * For further questions about DWARF, please contact us at
 * <dwarf@augmentedreality.de>
 * For questions about this particular service, contact
 * Daniel Pustka <pustka@in.tum.de>
 *
 * $Id: DerivThread.h,v 1.1 2004/08/14 23:32:39 pustka Exp $
 */

/**
 * @file DerivThread.h
 * @brief A thread that computes derivations
 *
 * @author Daniel Pustka <pustka@in.tum.de>
 * @ingroup PoseTool
 */

#ifndef __DerivThread_H_
#define __DerivThread_H_

// autoconf generated file with configuration information
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <PoseEkf.h>
#include <qthread.h>
#include "PoseSequence.h"

class DerivThread : public QThread
   {
   protected:
      PoseSequence& m_Destination;
      PoseSequenceSubCollection m_Sequences;

      QObject* m_pReceiver;
      QEvent* m_pEvent;
      int m_nDerivation;

      volatile bool m_bQuit;

      void run();

   public:
      DerivThread( PoseSequence& Destination, const PoseSequenceSubCollection& Sources,
         int nDerivation, QObject* pReceiver, QEvent* pEvent );

      void Quit()
         { m_bQuit = true; }
   };

#endif //__DerivThread_H_
