/* ========================================================================
 * Copyright (C) 2000-2004  Technische Universitaet Muenchen
 * License: GPL v2 (or later)
 *
 * For further information please contact the DWARF team at
 * <dwarf-gnu@augmentedreality.de>
 *
 * DerivWindowImpl.h
 * Daniel Pustka <pustka@in.tum.de> May 2004
 * Distributed Wearable Augmented Reality Framework - www.augmentedreality.de
 * For documentation and instructions, see the DWARF documentation website at
 * http://wwwbruegge.in.tum.de/projects/dwarf/doc/
 *
 * For further questions about DWARF, please contact us at
 * <dwarf@augmentedreality.de>
 * For questions about this particular service, contact
 * Daniel Pustka <pustka@in.tum.de>
 *
 * $Id: DerivWindowImpl.cpp,v 1.1 2004/08/14 23:32:39 pustka Exp $
 */

#include "DerivWindowImpl.h"
#include "PoseSequenceView.h"
#include <qslider.h>
#include <qlineedit.h>
#include <debug.h>

class DerivThreadFinishedEvent : public QEvent
   {
   public:
      static const int g_nEvID = QEvent::User + 167;
      int m_nThreadID;

      DerivThreadFinishedEvent( int nThreadID )
         : QEvent( static_cast<QEvent::Type>( g_nEvID ) )
         , m_nThreadID( nThreadID )
         {}

      virtual ~DerivThreadFinishedEvent() {}
   };


DerivWindowImpl::DerivWindowImpl( QWidget* pParent, const char* pszName )
   : DerivWindow( pParent, pszName )
   , m_pDestinationSequence( 0 )
   , m_nThreadID( 0 )
   , m_nDerivation( 0 )
   {
   setWFlags( Qt::WDestructiveClose );
   }


DerivWindowImpl::~DerivWindowImpl()
   {
   // unreference sequences
   StopThread();
   SetSequences( PoseSequenceSubCollection() );
   }


void DerivWindowImpl::SetSequences( const PoseSequenceSubCollection& Seqs )
   {
   // disconnect and unreference old sequences
   for ( unsigned i = 0; i < m_Sequences.size(); i++ )
      {
      disconnect( m_Sequences[ i ], SIGNAL( Changed() ), this, SLOT( SequenceChanged() ) );
      m_Sequences[ i ]->RemoveRef();
      }

   // copy sequence pointers
   m_Sequences = Seqs;

   if ( m_Sequences.empty() )
      return;

   std::string sName = "Drv";

   // connect and reference sequences
   for ( unsigned i = 0; i < m_Sequences.size(); i++ )
      {
      connect( m_Sequences[ i ], SIGNAL( Changed() ), this, SLOT( SequenceChanged() ) );
      m_Sequences[ i ]->AddRef();
      sName += "-" + m_Sequences[ i ]->GetName();
      }

   // create a new sequence for the result
   sName = g_pPoseSequenceCollection->CreateNewName( sName.c_str() );
   m_pDestinationSequence = g_pPoseSequenceCollection->AddSequence( sName.c_str() );

   SliderChanged( m_nDerivation );
   }


void DerivWindowImpl::SliderChanged( int nValue )
   {
   // update position
   m_nDerivation = nValue;
   StartThread();
   }


void DerivWindowImpl::SequenceChanged()
   {
   StartThread();
   }


bool DerivWindowImpl::event( QEvent* pEvent )
   {
   if ( pEvent->type() == DerivThreadFinishedEvent::g_nEvID )
      {
      DEBUGSTREAM( 20, "DerivWindow received finished() notification from DerivThread" );
      if ( static_cast<DerivThreadFinishedEvent*>( pEvent )->m_nThreadID == m_nThreadID )
         {
         StopThread();
         m_pDestinationSequence->SignalChanged();
         }
      else
         { DEBUGSTREAM( 20, "...from old thread" ); }
      return true;
      }
   else
      return DerivWindow::event( pEvent );
   }


void DerivWindowImpl::StartThread()
   {
   if ( m_Sequences.empty() )
      return;

   StopThread();
   m_pThread = std::auto_ptr<DerivThread>( new DerivThread( *m_pDestinationSequence, m_Sequences,
      m_nDerivation, this, new DerivThreadFinishedEvent( ++m_nThreadID ) ) );
   m_pThread->start();
   }


void DerivWindowImpl::StopThread()
   {
   if ( m_pThread.get() )
      {
      m_pThread->Quit();
      m_pThread->wait();
      m_pThread.reset();
      }
   }
