/* ========================================================================
 * Copyright (C) 2000-2004  Technische Universitaet Muenchen
 * License: GPL v2 (or later)
 *
 * For further information please contact the DWARF team at
 * <dwarf-gnu@augmentedreality.de>
 *
 * DerivWindowImpl.h
 * Daniel Pustka <pustka@in.tum.de> May 2004
 * Distributed Wearable Augmented Reality Framework - www.augmentedreality.de
 * For documentation and instructions, see the DWARF documentation website at
 * http://wwwbruegge.in.tum.de/projects/dwarf/doc/
 *
 * For further questions about DWARF, please contact us at
 * <dwarf@augmentedreality.de>
 * For questions about this particular service, contact
 * Daniel Pustka <pustka@in.tum.de>
 *
 * $Id: DerivWindowImpl.h,v 1.1 2004/08/14 23:32:39 pustka Exp $
 */

/**
 * @file DerivWindowImpl.h
 * @brief Implements the DerivWindow
 *
 * @author Daniel Pustka <pustka@in.tum.de>
 * @ingroup PoseTool
 */

#ifndef __DerivWindowImpl_H_
#define __DerivWindowImpl_H_

// autoconf generated file with configuration information
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "uic_DerivWindow.h"
#include <memory>
#include "PoseSequence.h"
#include "DerivThread.h"

class QWidget;
class QSlider;
class QLineEdit;


class DerivWindowImpl : public DerivWindow
   {
   Q_OBJECT

   protected:
      std::auto_ptr<DerivThread> m_pThread;

      PoseSequenceSubCollection m_Sequences;
      PoseSequence* m_pDestinationSequence;

      int m_nThreadID;

      int m_nDerivation;

      void StartThread();
      void StopThread();

      bool event( QEvent* );

   public:
      DerivWindowImpl( QWidget* pParent = 0, const char* pszName = 0 );
      ~DerivWindowImpl();

      void SetSequences( const PoseSequenceSubCollection& Seqs );

   public slots:
      void SequenceChanged();
      void SliderChanged( int nValue );
   };

#endif //__DerivWindowImpl_H_
