/* ========================================================================
 * Copyright (C) 2000-2004  Technische Universitaet Muenchen
 * License: GPL v2 (or later)
 *
 * For further information please contact the DWARF team at
 * <dwarf-gnu@augmentedreality.de>
 *
 * EkfThread.h
 * Daniel Pustka <pustka@in.tum.de> May 2004
 * Distributed Wearable Augmented Reality Framework - www.augmentedreality.de
 * For documentation and instructions, see the DWARF documentation website at
 * http://wwwbruegge.in.tum.de/projects/dwarf/doc/
 *
 * For further questions about DWARF, please contact us at
 * <dwarf@augmentedreality.de>
 * For questions about this particular service, contact
 * Daniel Pustka <pustka@in.tum.de>
 *
 * $Id: EkfThread.h,v 1.1 2004/06/07 13:40:08 pustka Exp $
 */

/**
 * @file EkfThread.h
 * @brief A thread that runs an EKF
 *
 * @author Daniel Pustka <pustka@in.tum.de>
 * @ingroup PoseTool
 */

#ifndef __EkfThread_H_
#define __EkfThread_H_

// autoconf generated file with configuration information
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <PoseEkf.h>
#include <qthread.h>
#include "PoseSequence.h"

class EkfThread : public QThread
   {
   protected:
      PoseSequence& m_Destination;
      PoseSequenceSubCollection m_Sequences;

      PoseEKF::MotionModel m_MoMo;
      int m_nPredict;
      int m_nSampleInterval;

      QObject* m_pReceiver;
      QEvent* m_pEvent;

      volatile bool m_bQuit;

      void run();

   public:
      EkfThread( PoseSequence& Destination, const PoseSequenceSubCollection& Sources,
         const PoseEKF::MotionModel& MoMo, int nPredict, int nSampleInterval,
         QObject* pReceiver, QEvent* pEvent );

      void Quit()
         { m_bQuit = true; }
   };

#endif //__EkfWindow_H_
