/* ========================================================================
 * Copyright (C) 2000-2004  Technische Universitaet Muenchen
 * License: GPL v2 (or later)
 *
 * For further information please contact the DWARF team at
 * <dwarf-gnu@augmentedreality.de>
 *
 * EkfWindowImpl.h
 * Daniel Pustka <pustka@in.tum.de> May 2004
 * Distributed Wearable Augmented Reality Framework - www.augmentedreality.de
 * For documentation and instructions, see the DWARF documentation website at
 * http://wwwbruegge.in.tum.de/projects/dwarf/doc/
 *
 * For further questions about DWARF, please contact us at
 * <dwarf@augmentedreality.de>
 * For questions about this particular service, contact
 * Daniel Pustka <pustka@in.tum.de>
 *
 * $Id: EkfWindowImpl.h,v 1.1 2004/06/07 13:40:08 pustka Exp $
 */

/**
 * @file EkfWindowImpl.h
 * @brief Implements the EkfWindow
 *
 * @author Daniel Pustka <pustka@in.tum.de>
 * @ingroup PoseTool
 */

#ifndef __EkfWindowImpl_H_
#define __EkfWindowImpl_H_

// autoconf generated file with configuration information
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "uic_EkfWindow.h"
#include <PoseEkf.h>
#include <memory>
#include "PoseSequence.h"
#include "EkfThread.h"

class QWidget;
class QSlider;
class QLineEdit;


class EkfWindowImpl : public EKFWindow
   {
   Q_OBJECT

   protected:
      struct EditPair
         {
         QSlider* pSlider;
         QLineEdit* pEdit;
         EditPair( QSlider* s, QLineEdit* e )
            { pSlider = s; pEdit = e; }
         };
      std::vector<EditPair> m_Controls;

      PoseEKF::MotionModel m_MoMo;
      std::auto_ptr<EkfThread> m_pThread;

      PoseSequenceSubCollection m_Sequences;
      PoseSequence* m_pDestinationSequence;

      int m_nThreadID;

      void UpdateControlPair( int iControl, double fVal );
      void UpdateControls();
      void UpdateEnabled();

      void StartThread();
      void StopThread();

      bool event( QEvent* );

   public:
      EkfWindowImpl( QWidget* pParent = 0, const char* pszName = 0 );
      ~EkfWindowImpl();

      void SetSequences( const PoseSequenceSubCollection& Seqs );

   public slots:
      void SliderChanged();
      void ReturnPressed();
      void DerivationsChanged();
      void SomethingElseChanged();
      void SequenceChanged();
      void AutoTune();
      void PredictionError();
   };

#endif //__EkfWindow_H_
