/* ========================================================================
 * Copyright (C) 2000-2004  Technische Universitaet Muenchen
 * License: GPL v2 (or later)
 *
 * For further information please contact the DWARF team at
 * <dwarf-gnu@augmentedreality.de>
 *
 * InfWindowImpl.h
 * Daniel Pustka <pustka@in.tum.de> May 2004
 * Distributed Wearable Augmented Reality Framework - www.augmentedreality.de
 * For documentation and instructions, see the DWARF documentation website at
 * http://wwwbruegge.in.tum.de/projects/dwarf/doc/
 *
 * For further questions about DWARF, please contact us at
 * <dwarf@augmentedreality.de>
 * For questions about this particular service, contact
 * Daniel Pustka <pustka@in.tum.de>
 *
 * $Id: InfWindowImpl.cpp,v 1.1 2004/08/14 23:32:39 pustka Exp $
 */

#include "InfWindowImpl.h"
#include "PoseSequenceView.h"
#include <debug.h>

static const double g_fLn10 = 2.30258509299404568401; // ln( 10 ) - used to compute 10^x from e^x

class InfThreadFinishedEvent : public QEvent
   {
   public:
      static const int g_nEvID = QEvent::User + 165;
      int m_nThreadID;

      InfThreadFinishedEvent( int nThreadID )
         : QEvent( static_cast<QEvent::Type>( g_nEvID ) )
         , m_nThreadID( nThreadID )
         {}

      virtual ~InfThreadFinishedEvent() {}
   };



InfWindowImpl::InfWindowImpl( QWidget* pParent, const char* pszName )
   : InfWindow( pParent, pszName )
   , m_pDestinationSequence( 0 )
   , m_nThreadID( 0 )
   {
   setWFlags( Qt::WDestructiveClose );
   }


InfWindowImpl::~InfWindowImpl()
   {
   // unreference sequences
   StopThread();
   SetSequences( PoseSequenceSubCollection() );
   }


void InfWindowImpl::SetSequences( const PoseSequenceSubCollection& Seqs )
   {
   // disconnect and unreference old sequences
   for ( unsigned i = 0; i < m_Sequences.size(); i++ )
      {
      disconnect( m_Sequences[ i ], SIGNAL( Changed() ), this, SLOT( SequenceChanged() ) );
      m_Sequences[ i ]->RemoveRef();
      }

   // copy sequence pointers
   m_Sequences = Seqs;

   if ( m_Sequences.empty() )
      return;

   std::string sName = "Inf";

   // connect and reference sequences
   for ( unsigned i = 0; i < m_Sequences.size(); i++ )
      {
      connect( m_Sequences[ i ], SIGNAL( Changed() ), this, SLOT( SequenceChanged() ) );
      m_Sequences[ i ]->AddRef();
      sName += "-" + m_Sequences[ i ]->GetName();
      }

   // create a new sequence for the result
   sName = g_pPoseSequenceCollection->CreateNewName( sName.c_str() );
   m_pDestinationSequence = g_pPoseSequenceCollection->AddSequence( sName.c_str() );

   StartThread();
   }


void InfWindowImpl::SequenceChanged()
   {
   StartThread();
   }


bool InfWindowImpl::event( QEvent* pEvent )
   {
   if ( pEvent->type() == InfThreadFinishedEvent::g_nEvID )
      {
      DEBUGSTREAM( 20, "InfWindow received finished() notification from InfThread" );
      if ( static_cast<InfThreadFinishedEvent*>( pEvent )->m_nThreadID == m_nThreadID )
         {
         StopThread();
         m_pDestinationSequence->SignalChanged();
         }
      else
         { DEBUGSTREAM( 20, "...from old thread" ); }
      return true;
      }
   else
      return InfWindow::event( pEvent );
   }


void InfWindowImpl::StartThread()
   {
   if ( m_Sequences.empty() )
      return;

   StopThread();
   m_pThread = std::auto_ptr<InfThread>( new InfThread( *m_pDestinationSequence, m_Sequences,
      this, new InfThreadFinishedEvent( ++m_nThreadID ) ) );
   m_pThread->start();
   }


void InfWindowImpl::StopThread()
   {
   if ( m_pThread.get() )
      {
      m_pThread->Quit();
      m_pThread->wait();
      m_pThread.reset();
      }
   }
