/* ========================================================================
 * Copyright (C) 2000-2004  Technische Universitaet Muenchen
 * License: GPL v2 (or later)
 *
 * For further information please contact the DWARF team at
 * <dwarf-gnu@augmentedreality.de>
 *
 * InvertThread.h
 * Daniel Pustka <pustka@in.tum.de> May 2004
 * Distributed Wearable Augmented Reality Framework - www.augmentedreality.de
 * For documentation and instructions, see the DWARF documentation website at
 * http://wwwbruegge.in.tum.de/projects/dwarf/doc/
 *
 * For further questions about DWARF, please contact us at
 * <dwarf@augmentedreality.de>
 * For questions about this particular service, contact
 * Daniel Pustka <pustka@in.tum.de>
 *
 * $Id: InvertThread.cpp,v 1.1 2004/08/14 23:32:39 pustka Exp $
 */

#include "InvertThread.h"
#include <debug.h>
#include <TimeUtil.h>
#include <qapplication.h>
#include <algorithm>
#include <ubitrack/EasyPoseData.h>

using DWARF::Time;
using DWARF::PoseData;
using DWARF::EasyPoseData;
using std::vector;


InvertThread::InvertThread( PoseSequence& Destination, const PoseSequenceSubCollection& Sources,
   QObject* pReceiver, QEvent* pEvent )
   : m_Destination( Destination )
   , m_Sequences( Sources )
   , m_pReceiver( pReceiver )
   , m_pEvent( pEvent )
   , m_bQuit( false )
   {
   }


void InvertThread::run()
   {
   DEBUGSTREAM( 20, "InvertThread started" );

   m_Sequences[ 0 ]->GetMutex().lock();

   // erase destination
   m_Destination.GetMutex().lock();
   m_Destination.clear();

   DEBUGSTREAM( 30, "InvertThread locked" );

   for ( PoseSequence::SequenceType::const_iterator it = m_Sequences[ 0 ]->begin();
      it != m_Sequences[ 0 ]->end() && !m_bQuit; it++ )
      m_Destination.push_back( EasyPoseData::inverse( *it ) );

   // unlock sequences
   m_Destination.GetMutex().unlock();
   m_Sequences[ 0 ]->GetMutex().unlock();

   // send event only when finished completely
   if ( !m_bQuit )
      {
      DEBUGSTREAM( 20, "InvertThread finished normally" );
      QApplication::postEvent( m_pReceiver, m_pEvent );
      }
   else
      {
      DEBUGSTREAM( 20, "InvertThread stopped unfinished" );
      delete m_pEvent;
      }
   }

