/* ========================================================================
 * Copyright (C) 2000-2004  Technische Universitaet Muenchen
 * License: GPL v2 (or later)
 *
 * For further information please contact the DWARF team at
 * <dwarf-gnu@augmentedreality.de>
 *
 * PoseSequenceView.h
 * Daniel Pustka <pustka@in.tum.de> May 2004
 * Distributed Wearable Augmented Reality Framework - www.augmentedreality.de
 * For documentation and instructions, see the DWARF documentation website at
 * http://wwwbruegge.in.tum.de/projects/dwarf/doc/
 *
 * For further questions about DWARF, please contact us at
 * <dwarf@augmentedreality.de>
 * For questions about this particular service, contact
 * Daniel Pustka <pustka@in.tum.de>
 *
 * $Id: PoseSequenceView.h,v 1.2 2004/08/14 23:32:39 pustka Exp $
 */

/**
 * @file PoseSequenceView.h
 * @brief Implements the PoseSequenceView
 *
 * @author Daniel Pustka <pustka@in.tum.de>
 * @ingroup PoseTool
 */

#ifndef __PoseSequenceView_H_
#define __PoseSequenceView_H_

#include <qscrollview.h>
#include <vector>
#include "PoseSequence.h"
#include "TimeUtil.h"

// autoconf generated file with configuration information
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

class QWidget;
class QPainter;
class QResizeEvent;

/**
 * Qt widget for displaying PoseData
 */
class PoseSequenceView : public QScrollView
   {
   Q_OBJECT

   protected:
      double m_fPixelsPerSecond;
      int m_nHeight;
      DWARF::Time m_FirstEventTime;
      double m_fDuration;
      double m_fMinValue;
      double m_fMaxValue;
      PoseSequenceSubCollection m_Sequences;

      bool m_bPosEnabled[ 3 ];
      bool m_bOriEnabled[ 4 ];
      bool m_bDrawVariance;
      bool m_bDrawDots;

      void UpdateMinMaxValues( bool bLocal = false );

      int ToX( const DWARF::Time& t )
         { DWARF::Time tr = t - m_FirstEventTime; return static_cast<int>(
            ( tr.seconds + tr.microseconds / 1000000.0 ) * m_fPixelsPerSecond ); }

      int ToY( double f )
         { return static_cast<int>(
            m_nHeight * ( 1.0 - ( f - m_fMinValue ) / ( m_fMaxValue - m_fMinValue ) ) ); }

      int QToY( double f )
         { return static_cast<int>( m_nHeight * ( 0.5 - f / 2.0 ) ); }

      void DrawPosition( QPainter* pPainter, const PoseSequence::SequenceType& Seq,
         int nClipX1, int nClipX2, int iPos, int iColor );

      void DrawOrientation( QPainter* pPainter, const PoseSequence::SequenceType& Seq,
         int nClipX1, int nClipX2, int iOri, int iColor );

   public:

      PoseSequenceView( QWidget* pParent = 0, const char* pszName = 0 );
      ~PoseSequenceView();

      void SetSequences( const PoseSequenceSubCollection& Seqs );

   signals:
      void CursorChanged( double, double );

   public slots:

      void enableX( bool b );
      void enableY( bool b );
      void enableZ( bool b );
      void enableRx( bool b );
      void enableRy( bool b );
      void enableRz( bool b );
      void enableRw( bool b );
      void enableVariance( bool b );
      void enableDots( bool b );

      void ZoomYLocal();
      void ZoomYGlobal();

      void ZoomXIn();
      void ZoomXOut();

      void SequenceChanged();

   protected:
      // Qt stuff
      void viewportResizeEvent( QResizeEvent* pEvent );
      void drawContents( QPainter* p, int nClipX, int nClipY, int nClipW, int nClipH );
      void contentsMouseMoveEvent( QMouseEvent* e );
      void contentsMousePressEvent( QMouseEvent* e );
      void contentsMouseReleaseEvent( QMouseEvent* e );
   };

#endif //__PoseSequenceView_H_
