/* ========================================================================
 * Copyright (C) 2000-2004  Technische Universitaet Muenchen
 * License: GPL v2 (or later)
 *
 * For further information please contact the DWARF team at
 * <dwarf-gnu@augmentedreality.de>
 *
 * TShiftThread.h
 * Daniel Pustka <pustka@in.tum.de> May 2004
 * Distributed Wearable Augmented Reality Framework - www.augmentedreality.de
 * For documentation and instructions, see the DWARF documentation website at
 * http://wwwbruegge.in.tum.de/projects/dwarf/doc/
 *
 * For further questions about DWARF, please contact us at
 * <dwarf@augmentedreality.de>
 * For questions about this particular service, contact
 * Daniel Pustka <pustka@in.tum.de>
 *
 * $Id: TShiftThread.cpp,v 1.1 2004/08/14 23:32:39 pustka Exp $
 */

#include "TShiftThread.h"
#include <debug.h>
#include <TimeUtil.h>
#include <qapplication.h>
#include <algorithm>
#include <ubitrack/EasyPoseData.h>

using DWARF::Time;
using DWARF::PoseData;
using DWARF::EasyPoseData;
using std::vector;


TShiftThread::TShiftThread( PoseSequence& Destination, const PoseSequenceSubCollection& Sources,
   int nShiftMs, QObject* pReceiver, QEvent* pEvent )
   : m_Destination( Destination )
   , m_Sequences( Sources )
   , m_pReceiver( pReceiver )
   , m_pEvent( pEvent )
   , m_nShiftMs( nShiftMs )
   , m_bQuit( false )
   {
   }


void TShiftThread::run()
   {
   DEBUGSTREAM( 20, "TShiftThread started" );

   m_Sequences[ 0 ]->GetMutex().lock();

   // erase destination
   m_Destination.GetMutex().lock();
   m_Destination.clear();
   m_Destination.SetDerivations( m_Sequences[ 0 ]->GetPosDerivation(),
      m_Sequences[ 0 ]->GetOriDerivation() );

   DEBUGSTREAM( 30, "TShiftThread locked" );

   Time ShiftTime = { std::abs( m_nShiftMs ) / 1000, ( std::abs( m_nShiftMs ) % 1000 ) * 1000 };
   for ( PoseSequence::SequenceType::const_iterator it = m_Sequences[ 0 ]->begin();
      it != m_Sequences[ 0 ]->end() && !m_bQuit; it++ )
      {
      m_Destination.push_back( *it );
      m_Destination.back().timestamp = m_nShiftMs < 0 ?
         ( it->timestamp - ShiftTime ) : ( it->timestamp + ShiftTime );
      }

   // unlock sequences
   m_Destination.GetMutex().unlock();
   m_Sequences[ 0 ]->GetMutex().unlock();

   // send event only when finished completely
   if ( !m_bQuit )
      {
      DEBUGSTREAM( 20, "TShiftThread finished normally" );
      QApplication::postEvent( m_pReceiver, m_pEvent );
      }
   else
      {
      DEBUGSTREAM( 20, "TShiftThread stopped unfinished" );
      delete m_pEvent;
      }
   }

